;////////////////////////////////////////////////////////////////////////////;
;/ asm.s                                                                    /;
;////////////////////////////////////////////////////////////////////////////;
	.include "\dev\gb\lib\global.s"

;////////////////////////////////////////////////////////////////////////////;
	.area	_HEADER (ABS)
;////////////////////////////////////////////////////////////////////////////;

;////////////////////////////////////////////////////////////////////////////;
;/ globals                                                                  /;
;////////////////////////////////////////////////////////////////////////////;
	;OAM tables (0x10 in properties = in use)
	_oam1		== 0xC700	;[64][4]	temporary OAM
	_oam2		== 0xC600	;[64][4]	OAM without scrolling
	_oam3		== 0xC000	;[64][4]	OAM with scrolling
	_oam4		== 0xFE00	;[64][4]	the hardware OAM

	_sprtmap	== 0xC800	;[18][32]	sprite overlay number (-1 = no sprite)
	;+4
	_attrmap	== 0xD800	;[18][32]	tile attribute
	;-3
	_tilemap	== 0xD000	;[18][32]	tile number
	;+2
	_gridmap	== 0xD400	;[18][32]

	_sx			== 0xFFC0
	_sy			== 0xFFC1
	_do_dma		== 0xFFC2
	_currentbank == 0xFFC3

;////////////////////////////////////////////////////////////////////////////;
	.area	_HEADER (ABS)
;////////////////////////////////////////////////////////////////////////////;
	.org	0x40		;VBL irq
	jp	.my_vbl_irq

	.org	0x48		;LCD irq
	jp	.my_lcd_irq
	
	.org	0x50		;TIM irq
	.globl	.WaveHandler
	jp		.WaveHandler
	;reti


	.org	0x3F00		;bkg fade
_fade_pal:
	.dw 32271,32304,32369,32402,32467,32531,32564,32629,32694,31735,27640
	.dw 24570,20475,16380,12285,8158,7999,7807,7678,5622,4591,2536,481
	.dw 480,448,416,416,384,384,352,352,320,288,288,256,256

;////////////////////////////////////////////////////////////////////////////;
	.area	_BSS
;////////////////////////////////////////////////////////////////////////////;
_fade_ptr:						;LSB ptr of bkg fade
	.ds 1

;////////////////////////////////////////////////////////////////////////////;
	.area	_CODE
;////////////////////////////////////////////////////////////////////////////;

;////////////////////////////////////////////////////////////////////////////;
;/ ,my_vbl_irq                                                              /;
;////////////////////////////////////////////////////////////////////////////;
.my_vbl_irq:
	push 	af
	push	bc
	push	de
	push 	hl


	;ei				;crashes
	

	;!!!!!!!!!! fix palette on top of screen
	;bkg fade
	ld		h, #>>_fade_pal
	.globl	_fade_ptr
	ld		l, #0
	ld		a, #128
	ldh		(.BCPS), a
	ld		a, (hl+)
	ldh		(.BCPD), a
	ld		a, (hl+)
	ldh		(.BCPD), a
	ld		a, l

	

	


	ld		a, (_do_dma)
	or		a, a
	jr		z, 8$
	call	.DoDMA
8$:



	xor		a, a
	ld		(_do_dma), a
	ld		(_fade_ptr), a

	ld		a, #8
	ldh		(.LYC), a


	ldh		a, (.LCDC)
	or		a, #0x20		;show window
	ldh		(.LCDC), a






	ldh		a, (.SCX)
	ld		b, a
	
	ld		c, #40
	ld		hl, #_oam2
	ld		de, #_oam3 ;0xFE00
	;di
4$:
	ld		a, (hl+)
	ld		(de), a
	inc		de
	ld		a, (hl+)
	sub		a, b
	ld		(de), a
	inc		de
	ld		a, (hl+)
	ld		(de), a
	inc		de
	ld		a, (hl+)
	ld		(de), a
	inc		de
	dec		c
	jr		nz, 4$

	ld		a, #0xC0
	call	0xFF82				;update sprites



	;read joypad
	.globl	.jpad
	call	.jpad
	or		a, a
	jr		z, 5$
	.globl	_joy
	;ld		b, a
	;ld		a, (_joy)
	;or		a, b
	ld		(_joy), a
5$:



	pop		hl
	pop		de
	pop		bc
	pop		af
	reti

;////////////////////////////////////////////////////////////////////////////;
;/ ,my_lcd_irq                                                              /;
;////////////////////////////////////////////////////////////////////////////;
.my_lcd_irq:
	push	af
	push	bc
	push	de
	push	hl


	;lyc
	ldh		a, (.LYC)
	add		a, #4
	ldh		(.LYC), a	



	;bkg fade
	ld		h, #>>_fade_pal
	.globl	_fade_ptr
	ld		a, (_fade_ptr)
	ld		l, a
1$:
	ldh		a, (.STAT)
	and		a, #2
	jr		nz, 1$

	ld		a, #128
	ldh		(.BCPS), a
	ld		a, (hl+)
	ldh		(.BCPD), a
	ld		a, (hl+)
	ldh		(.BCPD), a
	ld		a, l

	ld		(_fade_ptr), a






	;hide window on line 16
	cp		a, #16
	jr		nz, 2$
	ldh		a, (.LCDC)
	and		a, #0xDF
	ldh		(.LCDC), a
2$:



	;scroll on line 140
	ldh		a, (.LYC)
	cp		a, #140
	jr		nz, 6$
	ei
	call	.Scroll	
6$:


	;update sprites on line 144
	;ldh		a, (.LYC)
	;cp		a, #20
	;jr		nz, 5$
	;ei
	;call	.Sprites1
;5$:


	;update sprites on line 48
	;ldh		a, (.LYC)
	;cp		a, #
	;jr		nz, 7$
	;ei
	;call	.Sprites2
;7$:




	pop 	hl
	pop		de
	pop		bc
	pop		af
	reti


;////////////////////////////////////////////////////////////////////////////;
;/ .Scroll                                                                  /;
;////////////////////////////////////////////////////////////////////////////;
.Scroll:
	.globl	_puzzle
	ld		a, (_puzzle+24)		;puzzle.x * 8
	sla		a
	sla		a
	sla		a
	ld		b, a
	
	ldh		a, (.SCX)
	add		a, #12
	cp		a, b
	ret		z
	ldh		a, (.SCX)
	jr		c, 1$
	dec		a
	jr		3$
1$:
	inc		a
3$:
	ldh		(.SCX), a

	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .Sprites1                                                                /;
;////////////////////////////////////////////////////////////////////////////;
.Sprites1:
	ldh		a, (.SCX)
	ld		b, a

	ld		c, #40
	ld		hl, #_oam2
	ld		de, #0xFE00
4$:


10$:						;wait for mode 2
	ldh		a, (.STAT)
	and		a, #3
	cp		a, #2
	jr		nz, 10$

11$:						;wait for mode 0, we now know we are at the start of mode 0..more time..
	ldh	    a, (.STAT)
	and		a, #3
	cp		a, #0
	jr		nz, 11$


	ld		a, (hl+)
	ld		(de), a
	inc		de
	ld		a, (hl+)
	sub		a, b
	ld		(de), a
	inc		de
	ld		a, (hl+)
	ld		(de), a
	inc		de
	ld		a, (hl+)
	ld		(de), a
	inc		de
	dec		c
	jr		nz, 4$


	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .Sprites2                                                                /;
;////////////////////////////////////////////////////////////////////////////;
.Sprites2:
	ldh		a, (.SCX)
	ld		b, a

	ld		c, #20
	ld		hl, #_oam2+20*4
	ld		de, #0xFE00
4$:


10$:						;wait for mode 2
	ldh		a, (.STAT)
	and		a, #3
	cp		a, #2
	jr		nz, 10$

11$:						;wait for mode 0, we now know we are at the start of mode 0..more time..
	ldh	    a, (.STAT)
	and		a, #3
	cp		a, #0
	jr		nz, 11$


	ld		a, (hl+)
	ld		(de), a
	inc		de
	ld		a, (hl+)
	sub		a, b
	ld		(de), a
	inc		de
	ld		a, (hl+)
	ld		(de), a
	inc		de
	ld		a, (hl+)
	ld		(de), a
	inc		de
	dec		c
	jr		nz, 4$


	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .DoDMA                                                                   /;
;////////////////////////////////////////////////////////////////////////////;
.DoDMA:
	ld		a, #0
	ldh		(.VBK),	a
	
	ld		a, #>_tilemap
	ldh		(0x51), a
	ld		a, #5*32
	ldh		(0x52), a
	ld		a, #0x98
	ldh		(0x53), a
	ld		a, #5*32
	ldh		(0x54), a
	ld		a, #(10*32)/16-1
	ldh		(0x55), a

	ld		a, #1
	ldh		(.VBK),	a
	
	ld		a, #>_attrmap
	ldh		(0x51), a
	ld		a, #5*32
	ldh		(0x52), a
	ld		a, #0x98
	ldh		(0x53), a
	ld		a, #5*32
	ldh		(0x54), a
	ld		a, #(10*32)/16-1
	ldh		(0x55), a

	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _WaitVblDone                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_WaitVblDone::
	;ld		a, #1
	;ld		(_vbldone), a
;1$:
	;ld		a, (_vbldone)
	;or		a, a
	;jr		nz, 1$

	ldh		a, (.LY)
	cp		a, #144
	jr		nz, _WaitVblDone
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _ClearOAM1                                                               /;
;////////////////////////////////////////////////////////////////////////////;
_ClearOAM1::
	ld		hl, #_oam1
	ld		e, #40*4
	xor		a, a
1$:
	ld		(hl+), a
	dec		e
	jr		nz, 1$
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _CopyOAM1to2                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_CopyOAM1to2::
	push	bc
	ld		hl, #_oam1
	ld		de, #_oam2
	ld		c, #40*4
1$:
	ld		a, (hl+)
	ld		(de), a
	inc		de
	dec		c
	jr		nz, 1$
	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _InitMaps2                                                               /;
;////////////////////////////////////////////////////////////////////////////;
_InitMaps2::
	push	bc
	
	ld		hl, #_gridmap + 32*5
	ld		de, #32-24

	ld		b, #10
1$:
	ld		c, #24
2$:
	ld		a, (hl)		;_gridmap
	res		2, h
	ld		(hl), a		;_tilemap
	set		3, h
	ld		(hl), #0	;_attrmap
	res		4, h
	ld		(hl), #-1	;_sprtmap
	set		4, h
	res		3, h
	set		2, h
	inc		l
	dec		c
	jr		nz, 2$
	add		hl, de
	dec		b
	jr		nz, 1$
	
	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .RemoveSprite                                                            /;
;////////////////////////////////////////////////////////////////////////////;
.RemoveSprite:
	ld		a, (hl)
	cp		a, #-1
	ret		z			;-1

	push	hl
	ld		h, #>_oam1
	;sla		a
	;sla		a
	ld		l, a
	xor		a, a
	ld		(hl+), a	;y=0
	inc		l
	inc		l
	ld		(hl), a		;prop=0
	pop		hl

	ld		a, #-1
	ld		(hl), a
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .DrawTile                                                                /;
;////////////////////////////////////////////////////////////////////////////;
;hl=pos in sprtmap, b=character, c=attribute
.DrawTile:
	call	.RemoveSprite
	set		4, h		;sprtmap -> attrmap
	ld		(hl), c
	res		3, h		;attrmap -> tilemap
	ld		(hl), b
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ .DrawSprt                                                                /;
;////////////////////////////////////////////////////////////////////////////;
;hl=pos in sprtmap, b=character, c=attribute
.DrawSprt:


	;look for unused sprite
	ld		h, #>_oam1
	ld		l, #39*4+3
1$:
	ld		a, (hl-)
	and		a, #0x10
	jr		z, 2$
	dec		l
	dec		l
	dec		l
	jr		1$
2$:




	;set sprite prop, tile, x & y
	ld		a, c
	or		a, #0x10
	inc		hl
	ld		(hl-), a
	ld		a, b
	ld		(hl-), a
	;ld		a, (_sx)
	;add		a, #8*2
	;ld		(hl-), a
	;ld		a, (_sy)
	;add		a, #8*1
	;ld		(hl), a
	;ld		a, l
	;pop		hl
	;ld		(hl), a
	
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawCube                                                                /;
;////////////////////////////////////////////////////////////////////////////;
_DrawCube::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, (hl)
;		   __
;	123  /__/|
;	684 |  | |
;	795 |__|/
	
	ld		b, #0x01
	ld		hl, #32*0 + 0
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*0
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*0 + 1
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*0 + 2
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*1 + 2
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*2 + 2
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*1 + 0
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*2 + 0
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	call	.DrawTile

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	call	.DrawTile


	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer0                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer0::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x10
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer1                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer1::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x13
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer2                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer2::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x16
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer3                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer3::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x19
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer4                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer4::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x1C
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*0 + 2
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*2
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 2
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*2
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer5                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer5::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x21
	ld		hl, #32*0 + 0
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*0
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer6                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer6::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x23
	ld		hl, #32*0 + 0
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*0
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 0
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*0
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret

;////////////////////////////////////////////////////////////////////////////;
;/ _DrawPlayer7                                                             /;
;////////////////////////////////////////////////////////////////////////////;
_DrawPlayer7::
	push	bc
	lda		hl, 4(sp)
	ld		a, (hl+)
	ld		e, a
	ld		a, (hl+)
	ld		d, a
	ld		a, (hl+)
	ld		(_sx), a
	ld		a, (hl+)
	ld		(_sy), a
	ld		c, #0
	
	ld		b, #0x28
	ld		hl, #32*0 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*1 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*1
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*2 + 1
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*1
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*2
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	inc		b
	ld		hl, #32*0 + 2
	add		hl, de
	push	hl
	call	.DrawSprt
	ld		a, (_sx)
	add		a, #8*2
	ld		(hl-), a
	ld		a, (_sy)
	add		a, #8*0
	ld		(hl), a
	ld		a, l
	pop		hl
	ld		(hl), a

	pop		bc
	ret
