//////////////////////////////////////////////////////////////////////////////
// main.c                                                                   //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "sound.h"
#include "text.h"
#include "puzzle.h"
#include "rand.h"
#include "asm.h"
#include "font.h"
#include "display.h"
#include "tiles.h"
#include "normal.h"
#include "surprise.h"
#include "solid.h"
#include "player.h"

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define UP		0
#define DOWN	1
#define LEFT	2
#define RIGHT	3
#define PUSH	4

//////////////////////////////////////////////////////////////////////////////
// typedefs                                                                 //
//////////////////////////////////////////////////////////////////////////////
typedef void TDrawPlayer(void* a, unsigned char x, unsigned char y);

//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
UWORD tilepals[] = CGBPALS(tiles);
UWORD spritepals[] = CGBPALS(player);

unsigned int edit;						// editmode?
unsigned int cubes;						// cubes counted during drawing
unsigned int playerdir;
unsigned int joy;
unsigned int cnt;						// draw count

TDrawPlayer* drawplayer[8] = { DrawPlayer0, DrawPlayer1, DrawPlayer2, DrawPlayer3, DrawPlayer4, DrawPlayer5, DrawPlayer6, DrawPlayer7 };

//////////////////////////////////////////////////////////////////////////////
// DrawCubes                                                                //
//////////////////////////////////////////////////////////////////////////////
void DrawCubes()
{
	unsigned int x,y;
	
	cubes=0;
	cnt++;
	if (edit) playerdir = DOWN;
	
	for (y=0; y<DEPTH; y++)
	{
		for (x=0; x<WIDTH; x++)
		{
			unsigned int c = puzzle.map[y][x];
			
			if ((x == puzzle.x) && (y == puzzle.y))
			{
				if (!edit || (cnt&1))
				{
					Tab* tab = &tabs[y][x];
					drawplayer[playerdir](tab->a, tab->sx, tab->sy);
					c = 0;
				}
			}
			
			if (c)
			{
				Tab* tab = &tabs[y][x];
				if (c == 8) c = 7+8;	// special -> white special
				DrawCube(tab->a, tab->sx, tab->sy, c);
				cubes++;
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// DrawAll                                                                  //
//////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
	InitMaps2();
	ClearOAM1();
	DrawCubes();
	CopyOAM1to2();
	do_dma = 1;
	while (do_dma);
}

//////////////////////////////////////////////////////////////////////////////
// LoadTiles                                                                //
//////////////////////////////////////////////////////////////////////////////
void LoadTiles()
{
	BANK(TILES);
	VBK_REG=1;
	if (puzzle.special)
	{
		// surprise tiles
		set_bkg_data(0,10, surprise);
		set_sprite_data(0,10, surprise);
	}
	else
	{
		// solid tiles
		set_bkg_data(0,10, solid);
		set_sprite_data(0,10, solid);
	}

	VBK_REG=0;
	// normal tiles
	set_bkg_data(0,10, normal);
	set_sprite_data(0,10, normal);
	// player tiles
	set_sprite_data(16,32, player);
}

//////////////////////////////////////////////////////////////////////////////
// SoundStep                                                                //
//////////////////////////////////////////////////////////////////////////////
void SoundStep()
{
	switch (rand()&3)
	{
		case 0: PlaySound(STEP1); break;
		case 1: PlaySound(STEP2); break;
		case 2: PlaySound(STEP3); break;
		case 3: PlaySound(STEP4); break;
	}
}

//////////////////////////////////////////////////////////////////////////////
// Move                                                                     //
//////////////////////////////////////////////////////////////////////////////
int Move(int x1, int y1, int x2, int y2)
{
	unsigned int c1,c2;
	
	if ((x2<0) || (x2>=WIDTH) || (y2<0) || (y2>=DEPTH))	// dest in playfield?
		return 0;

	if (edit) return -1;

	if ((puzzle.map[y1][x1] | puzzle.map[y2][x2]) & 8)	// src or dest special block?
	{
		switch (puzzle.special)
		{
			case 0:	// solid block
				return 0;

			case 1:	// surprise
				break;
		}
		puzzle.whites--;
		//SndDiamond();
		return -1;
	}

	c2 = puzzle.map[y2][x2] & 7;		// dest color
	if (x1 == -1)						// player move
	{
		switch (c2)
		{
			case 7:						// absorb white
				puzzle.map[y2][x2] = 0;
				playerdir &= (PUSH-1);
				puzzle.whites--;
				PlaySound(PICKUP);
				return -1;
			
			case 0:						// free
				SoundStep();
				return -1;
			
			default:					// blocked
				return 0;
		}
	}
	
	c1 = puzzle.map[y1][x1] & 7;		// src color
	if (c1 & c2) return 0;				// collide?
	//if (c2)
	{
		PlaySound(MOVE1);
		WAIT_SOUND();
		PlaySound(MOVE2);
	}
	c2 |= c1;							// mix
	puzzle.map[y1][x1] = 0;				// clear src
	puzzle.map[y2][x2] = c2;			// set dest
	return -1;
}

//////////////////////////////////////////////////////////////////////////////
// HandleInput                                                              //
//////////////////////////////////////////////////////////////////////////////
int HandleInput()
{
	unsigned long idle=1000;
	
	if (!edit)
	do
	{
		if ((playerdir & PUSH) && (idle)) if (--idle == 0)
		{
			PlaySound(MOVE3);
			playerdir &= (PUSH-1); joy=0; return 0;
		}
	} while (!joy);

	if (joy & J_START)			// exit level
	{
		if (edit)
			Savepuzzle();		// save
		else
			PlaySound(EXPLODE);	// restart
		return -1;
	}
	else if (joy & J_SELECT)
	{
		if (edit)
		{
			if ((joy & J_UP) && (puzzle.whites < 20))	// whites++
				puzzle.whites++;
			if ((joy & J_DOWN) && (puzzle.whites > 0))	// whites-- (if 0 puzzle is cleared)
				puzzle.whites--;
		}
	}
	else
	{
		if (joy & J_A)
		{
			if (edit)
			{
				if (joy & J_B)							// clear block
					puzzle.map[puzzle.y][puzzle.x] = 0;
				else if ((cubes<35) && (puzzle.map[puzzle.y][puzzle.x] < 8))	// block++
					puzzle.map[puzzle.y][puzzle.x]++;
				cnt=1;
			}
		}
		else if (joy & J_B)
		{
			if (edit)
			{
				if (puzzle.map[puzzle.y][puzzle.x] > 0)	// block--
					puzzle.map[puzzle.y][puzzle.x]--;
				cnt=1;
			}
			else
			{
				PlaySound(NO1);
				LoadUndo();
			}
		}
		
		switch (joy & J_DIR)
		{
			case J_UP:
			{
				int c;
				if (puzzle.y>=1) if (puzzle.map[puzzle.y-1][puzzle.x]) { playerdir = PUSH+UP; DrawAll(); SaveUndo(); } else playerdir = UP;
				c = Move(-1,-1, puzzle.x,puzzle.y-1);
				if (!edit && !c) c = Move(puzzle.x,puzzle.y-1, puzzle.x,puzzle.y-2);
				if (c) puzzle.y--;
				break;
			}
			
			case J_DOWN:
			{
				int c;
				if (puzzle.y<DEPTH-1) if (puzzle.map[puzzle.y+1][puzzle.x]) { playerdir = PUSH+DOWN; DrawAll(); SaveUndo(); } else playerdir = DOWN;
				c = Move(-1,-1, puzzle.x,puzzle.y+1);
				if (!edit && !c) c = Move(puzzle.x,puzzle.y+1, puzzle.x,puzzle.y+2);
				if (c) puzzle.y++;
				break;
			}
			
			case J_LEFT:
			{
				int c;
				if (puzzle.x>=1) if (puzzle.map[puzzle.y][puzzle.x-1]) { playerdir = PUSH+LEFT; DrawAll(); SaveUndo(); } else playerdir = LEFT;
				c = Move(-1,-1, puzzle.x-1,puzzle.y);
				if (!edit && !c) c = Move(puzzle.x-1,puzzle.y, puzzle.x-2,puzzle.y);
				if (c) puzzle.x--;
				break;
			}
			
			case J_RIGHT:
			{
				int c;
				if (puzzle.x<WIDTH-1) if (puzzle.map[puzzle.y][puzzle.x+1]) { playerdir = PUSH+RIGHT; DrawAll(); SaveUndo(); } else playerdir = RIGHT;
				c = Move(-1,-1, puzzle.x+1,puzzle.y);
				if (!edit && !c) c = Move(puzzle.x+1,puzzle.y, puzzle.x+2,puzzle.y);
				if (c) puzzle.x++;
				break;
			}
		}
	}
	joy=0;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// PlayPuzzle                                                               //
//////////////////////////////////////////////////////////////////////////////
void PlayPuzzle()
{
	playerdir = DOWN;

	InitMaps1();
	VBK_REG=1; set_bkg_tiles(0,0,32,18, (void*)attrmap);
	VBK_REG=0; set_bkg_tiles(0,0,32,18, (void*)tilemap);
	
	LoadTiles();
	SCX_REG = (puzzle.x-1)*8;
	LYC_REG = 0;
	set_bkg_palette(0,8, tilepals);
	set_sprite_palette(0,8, spritepals);
	SHOW_SPRITES;
	set_interrupts(VBL_IFLAG | LCD_IFLAG | TIM_IFLAG);

	SaveUndo();
	joy = J_A;	// update
	
	while (1)
	{
		unsigned int s;

		VBK_REG=0;
		gotoxy(0,0); printf("White cubes left: %d ", puzzle.whites);
		
		if (!edit && (puzzle.whites == 0))
		{
			int i;
			
			WAIT_SOUND();
			PlaySound(SECRET);

			for (i=0; i<2; i++)
			{
				playerdir = LEFT; DrawAll(); delay(10);
				playerdir = UP; DrawAll(); delay(10);
				playerdir = RIGHT; DrawAll(); delay(10);
				playerdir = DOWN; DrawAll(); delay(10);
			}
			
			WAIT_SOUND();
			
			if (orgpuzzle < rampuzzles) NextPuzzle();
			break;
		}
		
		if (HandleInput()) break;

		DrawAll();
		
		delay(75);
		joy = 0;
	}
	
	set_interrupts(TIM_IFLAG);
}

//////////////////////////////////////////////////////////////////////////////
// LoadText                                                                 //
//////////////////////////////////////////////////////////////////////////////
void LoadText()		// tiles needed for text output etc!
{
	BANK(TILES);
	VBK_REG=0; set_bkg_data(0,128, tiles);	// load my tiles/font
}

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
void main()
{
	set_interrupts(0);
	LYC_REG = 0;
	STAT_REG = 0x40;		// LCD irq on LYC
	LCDC_REG &= 0xBF;		// window tilemap = bkg tilemap
	SHOW_BKG;
	InitSound();
	
	ShowBung(); PlaySound(SECRET); delay(500); while (!joypad()); clrscr();
	LoadText();
	PlaySound(FADEIN); Title();	 delay(500); while (!joypad()); PlaySound(FADEOUT); FadeDown(1); // clrscr();
	ShowBT();	PlaySound(EXPLODE); delay(500); while (!joypad()); clrscr();
	ShowLogo();	PlaySound(PICKUP); delay(500); while (!joypad()); clrscr();
	
	LoadText();
	orgpuzzle = puzzles;

	while (1)
	{
		SelectPuzzle();
		PlayPuzzle();
	}
}
