//////////////////////////////////////////////////////////////////////////////
// levels.cpp                                                               //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define WIDTH		8
#define DEPTH		8

//////////////////////////////////////////////////////////////////////////////
// typedefs                                                                 //
//////////////////////////////////////////////////////////////////////////////
typedef struct
{
	char id[3], crc;
	char title[20];
	char x,y;
	char whites;
	char special;
	char pad[4];
	unsigned char map[DEPTH][WIDTH];
} Puzzle;

//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
Puzzle puzzle;
FILE *fi;
int i;

//////////////////////////////////////////////////////////////////////////////
// Draw                                                                     //
//////////////////////////////////////////////////////////////////////////////
void Draw()
{
	textcolor(15);
	gotoxy(20, 4); cprintf("%-4d   : [%s]", i+1, puzzle.title);
	gotoxy(20, 5); cprintf("whites : %d   ", puzzle.whites);

	int cubes=0;
	for (int y=0; y<DEPTH; y++)
	{
		for (int x=0; x<WIDTH; x++)
		{
			gotoxy(x*2+32, y+9);
			switch (puzzle.map[y][x])
			{
				case 0: textcolor(0); break;
				case 1: textcolor(8+4); break;
				case 2: textcolor(8+2); break;
				case 3: textcolor(8+6); break;
				case 4: textcolor(8+1); break;
				case 5: textcolor(8+5); break;
				case 6: textcolor(8+3); break;
				case 7: textcolor(8+7); break;
				case 8: textcolor(8); break;
				default: textcolor(6); break;
			}
			cprintf("");
			if (puzzle.map[y][x])
				if ((y != puzzle.y) || (x != puzzle.x))
					cubes++;
		}
	}

	textcolor(15);
	gotoxy(20, 6); cprintf("cubes  : %d   ", cubes);
}

//////////////////////////////////////////////////////////////////////////////
// Valid                                                                    //
//////////////////////////////////////////////////////////////////////////////
int Valid()
{
	if (puzzle.id[0] != 'B') return 0;
	if (puzzle.id[1] != 'T') return 0;
	if (puzzle.id[2] != '!') return 0;
	return -1;
}

//////////////////////////////////////////////////////////////////////////////
// LoadLevel                                                                //
//////////////////////////////////////////////////////////////////////////////
int LoadLevel(int i)
{
	if (i < 0) return -1;
	fseek(fi, i*sizeof(Puzzle), SEEK_SET);
	fread(&puzzle, sizeof(Puzzle), 1, fi);
	if (feof(fi)) return -1;
	textcolor(15); gotoxy(20, 3); cprintf("Level loaded     ");
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// SaveLevel                                                                //
//////////////////////////////////////////////////////////////////////////////
int SaveLevel(int i)
{
	if (i < 0) return -1;
	fseek(fi, i*sizeof(Puzzle), SEEK_SET);
	fwrite(&puzzle, sizeof(Puzzle), 1, fi);
	textcolor(15); gotoxy(20, 3); cprintf("Level saved      ");
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main()
{
	textmode(C80);
	_setcursortype(_NOCURSOR);
	textbackground(1);
	clrscr();

	fi = fopen("..\\tab\\puzzles.bin", "r+b");
	if (!fi) return -1;

	if (LoadLevel(i=0)) return -1;

	while (i >= 0)
	{
		while (!kbhit())
		{
			int rnd = random(8);
			puzzle.map[puzzle.y][puzzle.x] ^= rnd; Draw();
			puzzle.map[puzzle.y][puzzle.x] ^= rnd; Draw();
		}

		textcolor(15); gotoxy(20, 3); cprintf("                 ");

		switch (getch())
		{
			case 27: i = -1; break;	// esc : quit
			case '!': SaveLevel(i); break;	// ! : save level
			case 'G': LoadLevel(i); break;	// home : reload level
			case 'I': if (LoadLevel(--i)) ++i; break;	// pgup : prev level
			case 'Q': if (LoadLevel(++i)) --i; break;	// pgdn : next level
			case '-': if (puzzle.whites > 0) puzzle.whites--; break;	// - : dec whites
			case '+': if (puzzle.whites < 20) puzzle.whites++; break;	// + : inc whites
			case ',': if (puzzle.map[puzzle.y][puzzle.x] > 0) puzzle.map[puzzle.y][puzzle.x]--; break;	// , : change cube
			case '.': if (puzzle.map[puzzle.y][puzzle.x] < 8) puzzle.map[puzzle.y][puzzle.x]++; break;	// . : change cube
			case 'K': if (puzzle.x > 0) puzzle.x--; break;
			case 'M': if (puzzle.x < 7) puzzle.x++; break;
			case 'H': if (puzzle.y > 0) puzzle.y--; break;
			case 'P': if (puzzle.y < 7) puzzle.y++; break;

			case 't':	// t : set title
			{
				char gs[64], *s = gs+2;
				textcolor(15);
				gotoxy(20, 4); cprintf("%-4d   : [                   ]", i+1);
				gotoxy(20, 4); cprintf("%-4d   : [", i+1);
				gs[0] = 18; cgets(gs);
				strcpy(puzzle.title, "");
				for (int i=0; i<(20-strlen(s))/2-1; i++) strcat(puzzle.title, " ");
				strcat(puzzle.title, "'");
				strcat(puzzle.title, s);
				strcat(puzzle.title, "'");
				while (strlen(puzzle.title) < 19) strcat(puzzle.title, " ");
				break;
			}

			case '[':	// [ : save and change order
			{
				Puzzle t = puzzle;
				if (LoadLevel(i-1)) break;	// prev level
				SaveLevel(i);
				puzzle = t; SaveLevel(--i);
				break;
			}

			case ']':	// ] : save and change order
			{
				Puzzle t = puzzle;
				if (LoadLevel(i+1)) break;	// next level
				SaveLevel(i);
				puzzle = t; SaveLevel(++i);
				break;
			}

			case 'f':
			{
				for (int y=0; y<DEPTH/2; y++)
				{
					for (int x=0; x<WIDTH; x++)
					{
						int t = puzzle.map[y][x];
						puzzle.map[y][x] = puzzle.map[7-y][x];
						puzzle.map[7-y][x] = t;
					}
				}
				break;
			}
		}
	}

	fclose(fi);
	textmode(LASTMODE);
	_setcursortype(_NORMALCURSOR);
	return 0;
}